@@ -7,20 +7,16 @@ module Main where
 
 import Criterion.Main
 import Text.Hamlet
-import Numeric (showInt)
 import qualified Data.ByteString.Lazy as L
 import qualified Text.Blaze.Html.Renderer.Utf8 as Utf8
 import Data.Monoid (mconcat)
 import Text.Blaze.Html5 (table, tr, td)
 import Text.Blaze.Html (toHtml)
 import Yesod.Core.Widget
-import Control.Monad.Trans.Writer
-import Control.Monad.Trans.RWS
-import Data.Functor.Identity
 import Yesod.Core.Types
-import Data.Monoid
-import Data.IORef
+import Data.Int
 
+main :: IO ()
 main = defaultMain
     [ bench "bigTable html" $ nf bigTableHtml bigTableData
     , bench "bigTable hamlet" $ nf bigTableHamlet bigTableData
@@ -35,6 +31,7 @@ main = defaultMain
     bigTableData = replicate rows [1..10]
     {-# NOINLINE bigTableData #-}
 
+bigTableHtml :: Show a => [[a]] -> Int64
 bigTableHtml rows = L.length $ Utf8.renderHtml $ ($ id) [hamlet|
 <table>
     $forall row <- rows
@@ -43,6 +40,7 @@ bigTableHtml rows = L.length $ Utf8.renderHtml $ ($ id) [hamlet|
                 <td>#{show cell}
 |]
 
+bigTableHamlet :: Show a => [[a]] ->  Int64
 bigTableHamlet rows = L.length $ Utf8.renderHtml $ ($ id) [hamlet|
 <table>
     $forall row <- rows
@@ -51,6 +49,7 @@ bigTableHamlet rows = L.length $ Utf8.renderHtml $ ($ id) [hamlet|
                 <td>#{show cell}
 |]
 
+bigTableWidget :: Show a => [[a]] -> IO Int64
 bigTableWidget rows = fmap (L.length . Utf8.renderHtml . ($ render)) (run [whamlet|
 <table>
     $forall row <- rows
@@ -64,6 +63,7 @@ bigTableWidget rows = fmap (L.length . Utf8.renderHtml . ($ render)) (run [whaml
     (_, GWData { gwdBody = Body x }) <- w undefined
     return x
 
-bigTableBlaze t = L.length $ Utf8.renderHtml $ table $ mconcat $ map row t
+bigTableBlaze :: Show a => [[a]] -> Int64
+bigTableBlaze t = L.length $ Utf8.renderHtml $ table $ Data.Monoid.mconcat $ map row t
   where
     row r = tr $ mconcat $ map (td . toHtml . show) r
